==> ecuacion-splib-web-1.73.5-sources.jar: jp/ecuacion/splib/web/controller/SplibBaseWithRecordController.java <==
package jp.ecuacion.splib.web.controller;

import jakarta.annotation.Nonnull;
import java.util.Set;
import jp.ecuacion.lib.core.exception.checked.AppException;
import jp.ecuacion.lib.core.exception.checked.CustomizedValidationAppException;
import jp.ecuacion.splib.web.advice.SplibExceptionHandlerData;
import jp.ecuacion.splib.web.advice.SplibExceptionHandler;
import jp.ecuacion.splib.web.enums.LoginStateEnum;
import jp.ecuacion.splib.web.exception.InputValidationException;
import jp.ecuacion.splib.web.form.SplibForm;
import jp.ecuacion.splib.web.util.TransactionTokenUtil;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;

public abstract class SplibBaseWithRecordController extends SplibBaseController {

  /**
   * その機能を表す名前。 form, recordなど各種クラスのprefix、urlの/xxx/searchなどの"xxx"部分などに使用。
   * 複数の機能で同一の機能名を持たせるのは不可。
   * <p>
   * "accGroup"のようにentity名と同一にするのが推奨。（実装上機能名及び各クラスにentity名が冠されることになりわかりやすいため）
   * ただし、実際問題、同一entityを主として使用するが、複数画面用意する必要がある場合などに名前を変える必要があるため、
   * "accGroupManagement"などentity名とは異なる名前も指定可能。 主要となるEntityは同一だが用途が異なるため画面を分けたい、などの場合には本機能名を分ける。
   * 異なるControllerで同一のfunctionNameを指定することは不可。 本fieldの値は、html側のtemplateにも渡され、同一のパラメータ名で使用される。
   * </p>
   */
  protected String functionName;
  
  /**
   * form直下のrecordのfield名。対応するentity名を使用する。 特にlist-edit
   * templateでは、form配下には一つのみのrecordを保持するルールのため、entity名と同一にしておくのが推奨。 （list-edit
   * templateではそれ以外のパターンをや流必要性がなく未実施のためサポート外）
   * <p>
   * java側では、spring側ではrecordNameが必要だが毎回書くのが面倒な際に自動補完する目的などで使用。
   * また本fieldの値は、html側のtemplateにも渡され、同一のパラメータ名で使用。
   * </p>
   * <p>
   * page-single-formの場合は、recordが存在しない場合もある。その場合は""を指定。（nullは不可）
   * </p>
   */
  @Nonnull
  protected String rootRecordName;
  
  /** functionNameを指定。 */
  public SplibBaseWithRecordController(LoginStateEnum loginState, @Nonnull String functionName) {
    super(loginState);
    this.functionName = functionName;
    this.rootRecordName = functionName;
  }

  /** functionName, rootRecordNameを指定。 */
  public SplibBaseWithRecordController(LoginStateEnum loginState, @Nonnull String functionName,
      String rootRecordName) {
    this(loginState, functionName);
    this.rootRecordName = rootRecordName == null ? "" : rootRecordName;
  }
  
  public String getFunctionName() {
    return functionName;
  }

  public String getRootRecordName() {
    return rootRecordName;
  }

  /**
   * 全処理に共通のmodelAttributeはSplibWebToolControllerAdviceに設定しているが、本controllerを使用する場合に必要なものはここで定義。
   * ちなみにmodel自体は、Controllerの保持する値に依存しないため、SplibControllerAdviceにてrequestに追加している。
   */
  @ModelAttribute
  private void setCommonParamsToModel(Model model) {
    model.addAttribute("functionName", functionName);
    model.addAttribute("rootRecordName", rootRecordName);
  }

  /**
   * 以下を実施。 設定項目の値に従ってpulldownの選択肢を動的に変更したい場合など、敢えてvalidation checkをしたくない場面もあるので、 validation
   * checkの要否を持たせている。
   * <ul>
   *    <li>transactionToken check</li>
   *    <li>validation check</li>
   * </ul>
   */
  protected void commonProc(SplibForm form, Model model, BindingResult result,
      boolean needsValidationCheck) throws InputValidationException, AppException {

    tokenCheck();

    if (needsValidationCheck) {
      validationCheck(form, result);
    }
  }

  protected void tokenCheck() throws CustomizedValidationAppException {
    // transactionToken check
    String tokenFromHtml =
        (String) request.getParameter(TransactionTokenUtil.SESSION_KEY_TRANSACTION_TOKEN);

    @SuppressWarnings("unchecked")
    Set<String> tokenSet = (Set<String>) request.getSession()
        .getAttribute(TransactionTokenUtil.SESSION_KEY_TRANSACTION_TOKEN);

    if (tokenSet != null && tokenFromHtml != null) {
      if (!tokenSet.contains(tokenFromHtml)) {
        if (this instanceof SplibListJpaController<?, ?>) {
          // tokenでのエラーは、edit画面でのエラーであってもlistに戻りたい（※）のでinfoからnextPageOnErrorを除去
          // （※）edti画面で新規登録 -> browser back -> 登録、とやると同じ内容のデータを2回登録できてしまうため。
          SplibExceptionHandlerData info = (SplibExceptionHandlerData) request
              .getAttribute(SplibExceptionHandler.INFO_FOR_ERROR_HANDLING);
          info.setNextPageOnError(null);
        }

        String msgId = "jp.ecuacion.splib.web.common.message.tokenInvalidate";
        throw new CustomizedValidationAppException(msgId);
      }

      tokenSet.remove(tokenFromHtml);
    }
  }

  private void validationCheck(SplibForm form, BindingResult result)
      throws InputValidationException {
    // input validation
    boolean hasNotEmptyError = false;
    hasNotEmptyError = form.hasNotEmptyError();

    if (hasNotEmptyError || (result != null && result.hasErrors())) {
      throw new InputValidationException(form);
    }
  }

  /**
   * validationチェックなし、エラー終了後の画面遷移なし、の場合は以下を使用。
   */
  protected void prepare(SplibForm form, Model model)
      throws InputValidationException, AppException {
    prepare(form, model, null, null, false);
  }

  /**
   * validationチェックなし、エラー終了後の画面遷移あり、の場合は以下を使用。
   */
  protected void prepare(SplibForm form, Model model, String nextPageOnError)
      throws InputValidationException, AppException {
    prepare(form, model, nextPageOnError, null, false);
  }

  /**
   * validationチェックあり、エラー終了後の画面遷移なし、の場合は以下を使用。
   */
  protected void prepare(SplibForm form, Model model, BindingResult result)
      throws InputValidationException, AppException {
    prepare(form, model, null, result, true);
  }

  /**
   * validationチェックあり、エラー終了後の画面遷移あり、の場合は以下を使用。
   */
  protected void prepare(SplibForm form, Model model, String nextPageOnError, BindingResult result)
      throws InputValidationException, AppException {
    prepare(form, model, nextPageOnError, result, true);
  }

  /**
   * 本メソッドは以下の処理を行う。 input validationは個別処理で書いても良いのだが、それも含めて1行でかけるものを用意した。
   * 
   * <p>
   * 1. AppExceptionに対するメッセージ出力をControllerの各処理で行うのではなく@ControllerAdviceで行うための準備
   * - @ControllerAdvice側でmodelをDIしても中身が空のため、controller側のものを保管しておく <br>
   * - エラー発生時の戻り先ページを設定しておく
   * </p>
   * <p>
   * commonProcの実行。
   * </p>
   */
  private void prepare(SplibForm form, Model model, String nextPageOnError, BindingResult result,
      boolean needsValidationCheck) throws InputValidationException, AppException {

    // 別画面に遷移した場合は使わない場合もあるが、同一画面に遷移する場合の同一formは自動で設定しておく
    model.addAttribute(functionName + "Form", form);

    // エラー処理用の準備
    SplibExceptionHandlerData info = new SplibExceptionHandlerData(this, form, nextPageOnError);
    request.setAttribute(SplibExceptionHandler.INFO_FOR_ERROR_HANDLING, info);
    
    commonProc(form, model, result, needsValidationCheck);
  }
}

==> ecuacion-splib-web-1.73.6-sources.jar: jp/ecuacion/splib/web/controller/SplibBaseWithRecordController.java <==
package jp.ecuacion.splib.web.controller;

import jakarta.annotation.Nonnull;
import java.util.Set;
import jp.ecuacion.lib.core.exception.checked.AppException;
import jp.ecuacion.lib.core.exception.checked.CustomizedValidationAppException;
import jp.ecuacion.splib.web.advice.SplibExceptionHandlerData;
import jp.ecuacion.splib.web.advice.SplibExceptionHandler;
import jp.ecuacion.splib.web.enums.LoginStateEnum;
import jp.ecuacion.splib.web.exception.InputValidationException;
import jp.ecuacion.splib.web.form.SplibForm;
import jp.ecuacion.splib.web.util.TransactionTokenUtil;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;

public abstract class SplibBaseWithRecordController extends SplibBaseController {

  /**
   * その機能を表す名前。 form, recordなど各種クラスのprefix、urlの/xxx/searchなどの"xxx"部分などに使用。
   * 複数の機能で同一の機能名を持たせるのは不可。
   * <p>
   * "accGroup"のようにentity名と同一にするのが推奨。（実装上機能名及び各クラスにentity名が冠されることになりわかりやすいため）
   * ただし、実際問題、同一entityを主として使用するが、複数画面用意する必要がある場合などに名前を変える必要があるため、
   * "accGroupManagement"などentity名とは異なる名前も指定可能。 主要となるEntityは同一だが用途が異なるため画面を分けたい、などの場合には本機能名を分ける。
   * 異なるControllerで同一のfunctionNameを指定することは不可。 本fieldの値は、html側のtemplateにも渡され、同一のパラメータ名で使用される。
   * </p>
   */
  protected String functionName;
  
  /**
   * form直下のrecordのfield名。対応するentity名を使用する。 特にlist-edit
   * templateでは、form配下には一つのみのrecordを保持するルールのため、entity名と同一にしておくのが推奨。 （list-edit
   * templateではそれ以外のパターンをや流必要性がなく未実施のためサポート外）
   * <p>
   * java側では、spring側ではrecordNameが必要だが毎回書くのが面倒な際に自動補完する目的などで使用。
   * また本fieldの値は、html側のtemplateにも渡され、同一のパラメータ名で使用。
   * </p>
   * <p>
   * page-general-formsの場合は、recordが存在しない場合もある。その場合は""を指定。（nullは不可）
   * </p>
   */
  @Nonnull
  protected String rootRecordName;
  
  /** functionNameを指定。 */
  public SplibBaseWithRecordController(LoginStateEnum loginState, @Nonnull String functionName) {
    super(loginState);
    this.functionName = functionName;
    this.rootRecordName = functionName;
  }

  /** functionName, rootRecordNameを指定。 */
  public SplibBaseWithRecordController(LoginStateEnum loginState, @Nonnull String functionName,
      String rootRecordName) {
    this(loginState, functionName);
    this.rootRecordName = rootRecordName == null ? "" : rootRecordName;
  }
  
  public String getFunctionName() {
    return functionName;
  }

  public String getRootRecordName() {
    return rootRecordName;
  }

  /**
   * 全処理に共通のmodelAttributeはSplibWebToolControllerAdviceに設定しているが、本controllerを使用する場合に必要なものはここで定義。
   * ちなみにmodel自体は、Controllerの保持する値に依存しないため、SplibControllerAdviceにてrequestに追加している。
   */
  @ModelAttribute
  private void setCommonParamsToModel(Model model) {
    model.addAttribute("functionName", functionName);
    model.addAttribute("rootRecordName", rootRecordName);
  }

  /**
   * 以下を実施。 設定項目の値に従ってpulldownの選択肢を動的に変更したい場合など、敢えてvalidation checkをしたくない場面もあるので、 validation
   * checkの要否を持たせている。
   * <ul>
   *    <li>transactionToken check</li>
   *    <li>validation check</li>
   * </ul>
   */
  protected void commonProc(SplibForm form, Model model, BindingResult result,
      boolean needsValidationCheck) throws InputValidationException, AppException {

    tokenCheck();

    if (needsValidationCheck) {
      validationCheck(form, result);
    }
  }

  protected void tokenCheck() throws CustomizedValidationAppException {
    // transactionToken check
    String tokenFromHtml =
        (String) request.getParameter(TransactionTokenUtil.SESSION_KEY_TRANSACTION_TOKEN);

    @SuppressWarnings("unchecked")
    Set<String> tokenSet = (Set<String>) request.getSession()
        .getAttribute(TransactionTokenUtil.SESSION_KEY_TRANSACTION_TOKEN);

    if (tokenSet != null && tokenFromHtml != null) {
      if (!tokenSet.contains(tokenFromHtml)) {
        if (this instanceof SplibListJpaController<?, ?>) {
          // tokenでのエラーは、edit画面でのエラーであってもlistに戻りたい（※）のでinfoからnextPageOnErrorを除去
          // （※）edti画面で新規登録 -> browser back -> 登録、とやると同じ内容のデータを2回登録できてしまうため。
          SplibExceptionHandlerData info = (SplibExceptionHandlerData) request
              .getAttribute(SplibExceptionHandler.INFO_FOR_ERROR_HANDLING);
          info.setNextPageOnError(null);
        }

        String msgId = "jp.ecuacion.splib.web.common.message.tokenInvalidate";
        throw new CustomizedValidationAppException(msgId);
      }

      tokenSet.remove(tokenFromHtml);
    }
  }

  private void validationCheck(SplibForm form, BindingResult result)
      throws InputValidationException {
    // input validation
    boolean hasNotEmptyError = false;
    hasNotEmptyError = form.hasNotEmptyError();

    if (hasNotEmptyError || (result != null && result.hasErrors())) {
      throw new InputValidationException(form);
    }
  }

  /**
   * validationチェックなし、エラー終了後の画面遷移なし、の場合は以下を使用。
   */
  protected void prepare(SplibForm form, Model model)
      throws InputValidationException, AppException {
    prepare(form, model, null, null, false);
  }

  /**
   * validationチェックなし、エラー終了後の画面遷移あり、の場合は以下を使用。
   */
  protected void prepare(SplibForm form, Model model, String nextPageOnError)
      throws InputValidationException, AppException {
    prepare(form, model, nextPageOnError, null, false);
  }

  /**
   * validationチェックあり、エラー終了後の画面遷移なし、の場合は以下を使用。
   */
  protected void prepare(SplibForm form, Model model, BindingResult result)
      throws InputValidationException, AppException {
    prepare(form, model, null, result, true);
  }

  /**
   * validationチェックあり、エラー終了後の画面遷移あり、の場合は以下を使用。
   */
  protected void prepare(SplibForm form, Model model, String nextPageOnError, BindingResult result)
      throws InputValidationException, AppException {
    prepare(form, model, nextPageOnError, result, true);
  }

  /**
   * 本メソッドは以下の処理を行う。 input validationは個別処理で書いても良いのだが、それも含めて1行でかけるものを用意した。
   * 
   * <p>
   * 1. AppExceptionに対するメッセージ出力をControllerの各処理で行うのではなく@ControllerAdviceで行うための準備
   * - @ControllerAdvice側でmodelをDIしても中身が空のため、controller側のものを保管しておく <br>
   * - エラー発生時の戻り先ページを設定しておく
   * </p>
   * <p>
   * commonProcの実行。
   * </p>
   */
  private void prepare(SplibForm form, Model model, String nextPageOnError, BindingResult result,
      boolean needsValidationCheck) throws InputValidationException, AppException {

    // 別画面に遷移した場合は使わない場合もあるが、同一画面に遷移する場合の同一formは自動で設定しておく
    model.addAttribute(functionName + "Form", form);

    // エラー処理用の準備
    SplibExceptionHandlerData info = new SplibExceptionHandlerData(this, form, nextPageOnError);
    request.setAttribute(SplibExceptionHandler.INFO_FOR_ERROR_HANDLING, info);
    
    commonProc(form, model, result, needsValidationCheck);
  }
}
